package org.act.soap.server.deploy;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;
import org.act.soap.server.config.ConfigException;
import org.act.soap.server.config.ConfigManager;
import org.act.soap.util.Constants;
import org.act.soap.util.FileUtil;
import org.act.soap.util.XMLUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.wsdd.Deployment;
import org.act.soap.wsdd.WSDDException;
import org.w3c.dom.Element;

public class DeployManager
    implements DeployEventManager {
    
    private static DeployManager mDeployManager = null;
    
    private static Log log = LogFactory.getLog(DeployManager.class.getName());
    
    private final FilenameFilter filter = new FilenameFilter() {
        public boolean accept (File dir, String filename) {
            if (!new File(dir, filename).isDirectory()) {
                return false;
            }
            if (null != filename) {
                final int len = filename.length();
                if (5 < len) {
                    String ext = filename.substring(len - 5, len);
                    return ext.equalsIgnoreCase(Constants.WSAR_FILE_SUFFIX);
                }
            }
            return false;
        }
    };
    
    private HashMap deployedServices = new HashMap();
    
    private AutoDeployer autoDeployer = null;
    
    private List listenerList = null;
    
    public static DeployManager getInstance () {
        if (null == mDeployManager) {
            mDeployManager = new DeployManager();
        }
        return mDeployManager;
    }
    
    public static void setCheckInterval (long interval) throws DeployException {
        try {
            ConfigManager.getInstance().setDeployerUpdateInterval(interval);
        }
        catch (ConfigException ex) {
            throw new DeployException(ex);
        }
    }
    
    public static long getCheckInterval () {
        return ConfigManager.getInstance().getDeployerUpdateInterval();
    }
    
    public static void setDeployDir (String dir) throws DeployException {
        if (null == dir) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        try {
            URL url = null;
            String deployPath = null;
            deployPath = new File(dir).getAbsolutePath();
            if (!deployPath.endsWith(File.separator)) {
                deployPath += File.separator;
            }
            url = new File(deployPath).toURL();
            ConfigManager.getInstance().setDeployPath(deployPath);
        }
        catch (java.net.MalformedURLException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_invalidDir",
                dir),
                                      ex);
        }
        catch (ConfigException cex) {
            throw new DeployException(Message.getMessage("ERR_SD_invalidDir",
                dir), cex);
        }
    }
    
    public static String getDeployDir () {
        return ConfigManager.getInstance().getDeployPath();
    }
    
    public static String getWorkingDir () {
        return ConfigManager.getInstance().getWorkingPath();
    }
    
    public static void setWorkingDir (String workingDir) throws DeployException {
        if (null == workingDir) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        try {
            String workingPath = null;
            URL url = null;
            workingPath = new File(workingDir).getAbsolutePath();
            if (!workingPath.endsWith(File.separator)) {
                workingPath += File.separator;
            }
            url = new File(workingPath).toURL();
            ConfigManager.getInstance().setWorkingPath(workingPath);
        }
        catch (java.net.MalformedURLException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_invalidDir",
                workingDir), ex);
        }
        catch (ConfigException cex) {
            throw new DeployException(Message.getMessage("ERR_SD_invalidDir",
                workingDir), cex);
        }
    }
    
    private static String getServiceNameByWsarName (String wsarName) {
        return wsarName.substring(0,
                                  wsarName.lastIndexOf(Constants.
            WSAR_FILE_SUFFIX));
    }
    
    private static void processDeployException (URL wsarWorkSpaceURL,
                                                String serviceName) throws
        DeployException {
        if ( (null == serviceName) || (null == wsarWorkSpaceURL)) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        try {
                        File fWorking = new File(wsarWorkSpaceURL.getPath());
            if (fWorking.exists()) {
                FileUtil.deleteRecursive(fWorking);
            }
        }
        catch (java.io.IOException ioEx) {
            throw new DeployException(Message.getMessage("ERR_SD_deploy11",
                serviceName), ioEx);         }
    }
    
    private DeployManager () {
        this.deployedServices = new HashMap();
    }
    
    public void init () {
                initDeployWorking();
                startAutoDeploy();
    }
    
    public DeployInfo deploy (File wsarFile) throws DeployException {
        FileInputStream in = null;
        if (null == wsarFile) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        try {
            in = new FileInputStream(wsarFile);
            return this.deploy(wsarFile.toURL(), in);
        }
        catch (MalformedURLException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_deploy10",
                wsarFile.toString()), ex);
        }
        catch (java.io.IOException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_deploy10",
                wsarFile.toString()), ex);
        }
        finally {
            try {
                if (null != in) {
                    in.close();
                }
            }
            catch (IOException ex) {
                throw new DeployException(ex);
            }
        }
    }
    
    public DeployInfo deploy (URL source, InputStream wsarContent) throws
        DeployException {
        URL wsarWorkSpaceURL = null;         String serviceName = null;         if ( (null == source) || (null == wsarContent)) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        try {
            Deployment wsddDeployment = null;
            File serviceConfFile = null;
            Element element = null;
            DeployInfo deployInfo = null;
                        String wsarFileName = new File(source.getPath()).getName();
                        if ( ( -1 == wsarFileName.lastIndexOf(Constants.WSAR_FILE_SUFFIX)) ||
                (0 == wsarFileName.lastIndexOf(Constants.WSAR_FILE_SUFFIX))) {
                throw new DeployException(Message.getMessage(
                    "ERR_SD_badServiceName01",
                    source.toString()));
            }
                        serviceName = getServiceNameByWsarName(wsarFileName);
            wsarWorkSpaceURL = new File(new File(DeployManager.getWorkingDir()),
                                        wsarFileName).toURL();
                        if (null != deployedServices.get(serviceName)) {
                throw new DeployException(Message.getMessage("ERR_SD_deploy12",
                    serviceName));
            }
                                    FileUtil.inflate(wsarContent, wsarWorkSpaceURL.getPath());
                        serviceConfFile = new File(wsarWorkSpaceURL.getPath(),
                                       Constants.META_INF_DIR_NAME +
                                       File.separator +
                                       Constants.WSDD_FILE_NAME);
            element = XMLUtils.getElementFromFile(serviceConfFile.
                                                  toString());
            wsddDeployment = new Deployment(element);
            wsddDeployment.setName( wsarFileName );
                        if (!wsddDeployment.getServiceName().equals(serviceName)) {
                                processDeployException(wsarWorkSpaceURL, serviceName);
                throw new DeployException(Message.getMessage(
                    "ERR_SD_badServiceName",
                    serviceName, wsddDeployment.getServiceName()));
            }
                        deployInfo = new DeployInfo();
            deployInfo.setServiceName(serviceName);
            deployInfo.setSource(source);
            deployInfo.setDestination(wsarWorkSpaceURL);
            deployInfo.setWSDDDeployment(wsddDeployment);
                                    this.notifyDeploy(deployInfo);
                        this.deployedServices.put(serviceName, deployInfo);
                        log.info(Message.getMessage("Info_SD_deploy1", serviceName));
            return deployInfo;
        }
        catch (MalformedURLException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_deploy10",
                source.toString()), ex);
        }
        catch (java.io.IOException ex) {
                                    processDeployException(wsarWorkSpaceURL, serviceName);
            throw new DeployException(Message.getMessage("ERR_SD_deploy10",
                serviceName), ex);
        }
        catch (WSDDException ex) {
                                    processDeployException(wsarWorkSpaceURL, serviceName);
            throw new DeployException(Message.getMessage("ERR_SD_deploy10",
                serviceName), ex);
        }
    }
    
    public boolean isServiceDeployed (String serviceName) {
        return null == serviceName ? false :
            deployedServices.containsKey(serviceName);
    }
    
    public void undeploy (String serviceName) throws DeployException {
        if (null == serviceName) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
                if (!this.deployedServices.containsKey(serviceName)) {
            return;
        }
                log.info(Message.getMessage("Info_SD_undeploy0", serviceName));
                try {
            DeployInfo deployInfo = (DeployInfo)this.deployedServices.get(
                serviceName);
            File file = new File(deployInfo.getDestination().getPath());
            if (file.exists()) {
                                FileUtil.deleteRecursive(file);
            }
                               this.deployedServices.remove(serviceName);
                        this.notifyUndeploy(deployInfo);
                        log.info(Message.getMessage("Info_SD_undeploy1", serviceName));
        }
        catch (java.io.IOException ex) {
            throw new DeployException(Message.getMessage("ERR_SD_undeploy00",
                serviceName), ex);
        }
    }
    
    public DeployInfo getDeployInfo (String serviceName) {
        return null == serviceName ? null :
            (DeployInfo)this.deployedServices.get(serviceName);
    }
    
    public HashMap getDeployedWSARs () {
        return (HashMap)this.deployedServices.clone();
    }
    
    public Deployment getWSDDDeployment (String serviceName) {
        DeployInfo deployInfo = null;
        if (null == serviceName) {
            return null;
        }
        deployInfo = (DeployInfo)this.deployedServices.get(
            serviceName);
        return null == deployInfo ? null : deployInfo.getWSDDDeployment();
    }
    
    public String getServiceWorkingPath (String serviceName) throws
        DeployException {
        DeployInfo deployInfo = null;
        if (null == serviceName) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        deployInfo = this.getDeployInfo(serviceName);
        if (null == deployInfo) {
            throw new DeployException(Message.getMessage(
                "ERR_SD_nullDeployInfo"));
        }
        return deployInfo.getDestination().getPath();
    }
    
    public void startAutoDeploy () {
        if (null == this.autoDeployer) {
            this.autoDeployer = new AutoDeployer(this);
        }
        this.autoDeployer.start();
    }
    
    public void stopAutoDeploy () {
        if (null != this.autoDeployer) {
            this.autoDeployer.stop();
        }
    }
    
    public boolean isAutoDeployStart () {
        return null == this.autoDeployer ? false : autoDeployer.isRunning();
    }
    
    
    public void addDeployEventListener (DeployEventListener listener) {
        if (null == listener) {
            return;
        }
        if (null == listenerList) {
            listenerList = new Vector();
        }
        listenerList.add(listener);
    }
    
    public void removeDeployEventListener (DeployEventListener listener) {
        if (null == listener) {
            return;
        }
        if (null != listenerList) {
            listenerList.remove(listener);
        }
    }
    
    public void notifyDeploy (DeployInfo deployInfo) throws DeployException {
        if (null == deployInfo) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        if (null != listenerList) {
            int notifiedNumber = 0;
            for (int i = 0; i < listenerList.size(); i++) {
                DeployEventListener listener =
                    (DeployEventListener) listenerList.get(i);
                try {
                    listener.onDeploy(deployInfo);
                    notifiedNumber++;
                }
                catch (DeployException ex) {
                                                            for (int j = 0; j < notifiedNumber; j++) {
                        listener.onUndeploy(deployInfo);
                    }
                    throw new DeployException(Message.getMessage(
                        "ERR_SD_notifyListener"),
                                              ex);
                }
            }
        }
    }
    
    public void notifyUndeploy (DeployInfo deployInfo) throws DeployException {
        if (null == deployInfo) {
            throw new DeployException(Message.getMessage("ERR_SD_NullArgument"));
        }
        if (null != listenerList) {
            for (int i = 0; i < listenerList.size(); i++) {
                DeployEventListener listener =
                    (DeployEventListener) listenerList.get(i);
                listener.onUndeploy(deployInfo);
            }
        }
    }
    
    
    private void initDeployWorking () {
        File fWorkingDir = new File(DeployManager.getWorkingDir());
        File[] wsarDirs = null;
        String serviceName = "";
        if (null == fWorkingDir) {
            return;
        }
                wsarDirs = fWorkingDir.listFiles(filter);
        if (null == wsarDirs) {
            return;
        }
        for (int i = 0; i < wsarDirs.length; i++) {
            try {
                Deployment wsddDeployment = null;
                File serviceConfFile = null;
                Element element = null;
                DeployInfo deployInfo = null;
                URL wsarDeploySpaceUrl = wsarDirs[i].toURL();
                                String strWsarFile = wsarDirs[i].toString().substring(wsarDirs[
                    i].
                    toString().lastIndexOf(File.separator) + 1);
                                serviceName = getServiceNameByWsarName(strWsarFile);
                                log.info(Message.getMessage("Info_SD_deploy0", serviceName));
                                serviceConfFile = new File(wsarDeploySpaceUrl.getPath(),
                                           Constants.META_INF_DIR_NAME +
                                           File.separator +
                                           Constants.WSDD_FILE_NAME);
                element = XMLUtils.getElementFromFile(serviceConfFile.
                    toString());
                wsddDeployment = new Deployment(element);
                wsddDeployment.setName( strWsarFile );
                                if (!wsddDeployment.getServiceName().equals(serviceName)) {
                    log.error(Message.getMessage("ERR_SD_badServiceName",
                                                 serviceName,
                                                 wsddDeployment.getServiceName()));
                    continue;
                }
                deployInfo = new DeployInfo();
                deployInfo.setServiceName(serviceName);
                                deployInfo.setSource(wsarDeploySpaceUrl);
                deployInfo.setDestination(wsarDeploySpaceUrl);
                deployInfo.setWSDDDeployment(wsddDeployment);
                                                this.notifyDeploy(deployInfo);
                                deployedServices.put(serviceName, deployInfo);
                                log.info(Message.getMessage("Info_SD_deploy1", serviceName));
            }
            catch (java.net.MalformedURLException ex) {
                log.error(ex.getMessage());
            }
            catch (WSDDException ex) {
                log.error(Message.getMessage("ERR_SD_badWSDD", serviceName), ex);
            }
            catch (DeployException ex) {
                log.error(ex.getMessage());
            }
            catch(Exception ex){
                log.error(ex.getMessage());;
            }
        }     }
}
